// Includes ////////////////////////////////////////////////////////////////

#include "UI.h"

// Static Variables ////////////////////////////////////////////////////////

UI::Dpi::GetDpiForWindowFunction UI::Dpi::GetDpiForWindow = nullptr;
UI::Dpi::AdjustWindowRectExForDpiFunction UI::Dpi::AdjustWindowRectExForDpi = nullptr;
UI::Dpi::GetSystemMetricsForDpiFunction UI::Dpi::GetSystemMetricsForDpi = nullptr;
bool UI::Dpi::loaded = false;

// Functions ///////////////////////////////////////////////////////////////

UI::Dpi::Dpi(const Window &window) :
	dpi(InitDpi(window))
{
}

////////////////////////////////////////////////////////////////////////////

UI::Dpi::Dpi(int dpi) :
	dpi(dpi)
{
	Load();
}

////////////////////////////////////////////////////////////////////////////

int UI::Dpi::Get() const
{
	return dpi;
}

////////////////////////////////////////////////////////////////////////////

int UI::Dpi::Scale(int value) const
{
	return value * dpi / 96;
}

////////////////////////////////////////////////////////////////////////////

bool UI::Dpi::AdjustWindowRect(RECT &rect, DWORD style, DWORD exStyle, bool menu) const
{
	if (AdjustWindowRectExForDpi)
		return AdjustWindowRectExForDpi(&rect, style, menu, exStyle, dpi);

	return AdjustWindowRectEx(&rect, style, menu, exStyle);
}

////////////////////////////////////////////////////////////////////////////

int UI::Dpi::GetSystemMetrics(int index) const
{
	if (GetSystemMetricsForDpi)
		return GetSystemMetricsForDpi(index, dpi);

	return ::GetSystemMetrics(index);
}

////////////////////////////////////////////////////////////////////////////

void UI::Dpi::Load() const
{
	if (loaded)
		return;

	loaded = true;

	if (!IsWindows10OrGreater())
		return;

	HMODULE user32 = LoadLibraryW(L"user32.dll");

	if (!user32)
		return;

	GetDpiForWindow = reinterpret_cast<GetDpiForWindowFunction>(GetProcAddress(user32, "GetDpiForWindow"));
	AdjustWindowRectExForDpi = reinterpret_cast<AdjustWindowRectExForDpiFunction>(GetProcAddress(user32, "AdjustWindowRectExForDpi"));
	GetSystemMetricsForDpi = reinterpret_cast<GetSystemMetricsForDpiFunction>(GetProcAddress(user32, "GetSystemMetricsForDpi"));
}

////////////////////////////////////////////////////////////////////////////

int UI::Dpi::InitDpi(const Window &window) const
{
	Load();

	if (GetDpiForWindow)
	{
		int dpi = GetDpiForWindow(window.GetHandle());

		if (dpi > 0)
			return dpi;
	}

	ScreenDC dc;
	return dc.GetDeviceCaps(LOGPIXELSX);
}

////////////////////////////////////////////////////////////////////////////
