#ifndef UI_SYSTEMTHEME_H
#define UI_SYSTEMTHEME_H

// Includes ////////////////////////////////////////////////////////////////

#include "UI_Common.h"

// Enums ///////////////////////////////////////////////////////////////////

namespace UI
{
	enum class PreferredAppMode
	{
		Default,
		AllowDark,
		ForceDark,
		ForceLight
	};
}

// Classes /////////////////////////////////////////////////////////////////

namespace UI
{
	class Window;
	class Control;

	class SystemTheme
	{
	public:

		////////////////////////////////////////////////////////////////////
		// Public //////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////

		// Functions ///////////////////////////////////////////////////////

		SystemTheme();
		bool IsHighContrast() const;
		bool IsDarkMode() const;
		bool SetDarkMode(const Window &window, bool enable) const;
		bool SetDarkMode(const Control &control, bool enable) const;

		////////////////////////////////////////////////////////////////////

	private:

		////////////////////////////////////////////////////////////////////
		// Private /////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////

		// Typedefs ////////////////////////////////////////////////////////

		typedef void (WINAPI *RtlGetNtVersionNumbersFunction)(DWORD *major, DWORD *minor, DWORD *build);
		typedef bool (WINAPI *ShouldAppsUseDarkModeFunction)();
		typedef PreferredAppMode (WINAPI *SetPreferredAppModeFunction)(PreferredAppMode);

		// Constants ///////////////////////////////////////////////////////

		static const int MinMajor;
		static const int MinMinor;
		static const int MinBuild;

		static const int DWMWA_USE_IMMERSIVE_DARK_MODE;
		static const int DWMWA_USE_IMMERSIVE_DARK_MODE_BEFORE_20H1;

		// Static Variables ////////////////////////////////////////////////

		static RtlGetNtVersionNumbersFunction RtlGetNtVersionNumbers;
		static ShouldAppsUseDarkModeFunction ShouldAppsUseDarkMode;
		static SetPreferredAppModeFunction SetPreferredAppMode;
		static bool loaded;

		// Functions ///////////////////////////////////////////////////////

		void Load() const;

		////////////////////////////////////////////////////////////////////
	};
}

////////////////////////////////////////////////////////////////////////////

#endif
